# Generated by Django 3.0.14 on 2023-03-30 02:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0006_auto_20230330_0203'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='registry',
            options={'ordering': ('name', '-active'), 'verbose_name_plural': 'Registries'},
        ),
        migrations.AlterField(
            model_name='registry',
            name='active',
            field=models.BooleanField(default=True),
        ),
    ]



# Generated by Django 3.0.14 on 2023-05-02 08:29

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0009_auto_20230331_0133'),
    ]

    operations = [
        migrations.CreateModel(
            name='TokenHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category', models.CharField(max_length=255)),
                ('token', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('date', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            options={
                'ordering': ('-date',),
            },
        ),
    ]



# Generated by Django 3.0.14 on 2023-05-02 08:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0011_auto_20230502_0839'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tokenhistory',
            name='category',
        ),
    ]



from django_filters import rest_framework as filters

from bcmr.models import *


# class RegistryFilter(filters.FilterSet):
#     class Meta:
#         model = Registry
#         fields = (
#             'owner',
#         )


class TokenFilter(filters.FilterSet):
    class Meta:
        model = Token
        fields = (
            'owner',
            'is_nft',
        )



# Generated by Django 3.0.14 on 2023-03-31 01:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0007_auto_20230330_0225'),
    ]

    operations = [
        migrations.AlterField(
            model_name='token',
            name='description',
            field=models.TextField(blank=True, default=''),
        ),
    ]



from django.contrib.postgres.fields import JSONField
from django.utils import timezone
from django.db import models

import uuid


class AuthToken(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    class Meta:
        verbose_name_plural = 'Auth Tokens'
    

class Token(models.Model):
    class Status(models.TextChoices):
        ACTIVE = 'active'
        INACTIVE = 'inactive'
        BURNED = 'burned'

    category = models.CharField(max_length=255, primary_key=True, unique=True)
    name = models.CharField(max_length=255)
    description = models.TextField(blank=True, default='')
    symbol = models.CharField(max_length=100)
    decimals = models.PositiveIntegerField(default=0)
    icon = models.ImageField(null=True, blank=True)
    is_nft = models.BooleanField(default=False)

    nft_description = models.CharField(max_length=255, blank=True, default='')
    nft_types = JSONField(default=dict)
    '''
        NFT_TYPES
        {
            "commitment_string_here": {
                "name": "NFT #0",
                "description": "",
                "fields": [],
                "uris": {
                    "icon": "https://gist.githubusercontent.com/mr-zwets/0e698a88323465686437b5e70a8ccf56/raw/Eo_circle_blue_number-0.svg"
                }
            }
        }
    '''
    date_created = models.DateTimeField(default=timezone.now)
    status = models.CharField(
        max_length=10,
        choices=Status.choices,
        default=Status.ACTIVE
    )
    owner = models.ForeignKey(
        AuthToken,
        related_name='tokens',
        on_delete=models.PROTECT,
        null=True,
        blank=True
    )

    class Meta:
        ordering = (
            'name',
            'symbol',
            'is_nft',
        )


class TokenHistory(models.Model):
    token = JSONField(default=dict) # see utils/record_token_history for data structure
    date = models.DateTimeField(default=timezone.now)

    class Meta:
        ordering = ('-date', )
        verbose_name_plural = 'Token Histories'


class Registry(models.Model):
    major = models.PositiveIntegerField(default=0) # incremented when an identity is removed
    minor = models.PositiveIntegerField(default=0) # incremented when an identity is added
    patch = models.PositiveIntegerField(default=0) # incremented when an identity is modified
    name = models.CharField(max_length=255)
    description = models.CharField(max_length=255, null=True, blank=True)
    active = models.BooleanField(default=True)
    date_created = models.DateTimeField(default=timezone.now)
    latest_revision = models.DateTimeField(default=timezone.now)
    tokens = models.ManyToManyField(Token)
    # owner = models.ForeignKey(
    #     AuthToken,
    #     related_name='registries',
    #     on_delete=models.PROTECT,
    #     null=True,
    #     blank=True
    # )

    class Meta:
        verbose_name_plural = 'Registries'
        ordering = ('name', '-active', )

    def save(self, *args, **kwargs):
        self.latest_revision = timezone.now()
        super(Registry, self).save(*args, **kwargs)



# Generated by Django 3.0.14 on 2023-05-02 08:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0010_tokenhistory'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='tokenhistory',
            options={'ordering': ('-date',), 'verbose_name_plural': 'Token Histories'},
        ),
    ]



# Generated by Django 3.1.6 on 2023-02-27 06:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0004_auto_20230214_0401'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='registry',
            name='owner',
        ),
        migrations.AddField(
            model_name='registry',
            name='active',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.0.14 on 2023-03-30 02:03

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bcmr', '0005_auto_20230227_0612'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='token',
            options={'ordering': ('name', 'symbol', 'is_nft')},
        ),
        migrations.AddField(
            model_name='token',
            name='is_nft',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='token',
            name='nft_description',
            field=models.CharField(default='', max_length=255),
        ),
        migrations.AddField(
            model_name='token',
            name='nft_types',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name='token',
            name='description',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
    ]



from rest_framework import serializers

from django.conf import settings

from bcmr.models import *


class EmptySerializer(serializers.Serializer):
    pass


class CashTokenSerializer(serializers.ModelSerializer):
    class Meta:
        model = Token
        fields = (
            'category',
            'name',
            'symbol',
            'description',
            'decimals',
            'icon',
            'is_nft',
            'nft_description',
            'nft_types',
            'status',
            'date_created',
            'owner',
        )
        read_only_fields = (
            'date_created',
            'owner',
        )


class MainTokenSerializer(serializers.ModelSerializer):
    token = serializers.SerializerMethodField()
    uris = serializers.SerializerMethodField()

    class Meta:
        model = Token
        fields = (
            'name',
            'description',
            'token',
            'status',
            'uris',
        )

    def get_token(self, obj):
        result = {
            'category': obj.category,
            'symbol': obj.symbol,
            'decimals': obj.decimals 
        }

        if obj.is_nft:
            result['nfts'] = {
                'description': obj.nft_description,
                'fields': {},
                'parse': {
                    'bytecode': '',
                    'types': obj.nft_types
                }
            }
        
        return result

    def get_uris(self, obj):
        if obj.icon:
            return {
                'icon': f'{settings.DOMAIN}{obj.icon.url}'
            }
        return {}


class RegistrySerializer(serializers.ModelSerializer):
    class Meta:
        model = Registry
        fields = (
            'id',
            'name',
            'description',
            'tokens',
            # 'owner',
        )
        read_only_fields = (
            # 'owner',
            'id',
        )


# class NoOwnerRegistrySerializer(serializers.ModelSerializer):
#     class Meta:
#         model = Registry
#         fields = (
#             'id',
#             'name',
#             'description',
#             'tokens',
#         )
#         read_only_fields = (
#             'id',
#         )


class TokenHistorySerializer(serializers.ModelSerializer):
    name = serializers.SerializerMethodField()
    description = serializers.SerializerMethodField()
    token = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()
    uris = serializers.SerializerMethodField()

    class Meta:
        model = TokenHistory
        fields = (
            'name',
            'description',
            'token',
            'status',
            'uris',
        )

    def get_name(self, obj):
        return obj.token['name']

    def get_description(self, obj):
        return obj.token['description']

    def get_status(self, obj):
        return obj.token['status']

    def get_uris(self, obj):
        if obj.token['icon']:
            return {
                'icon': f"{settings.DOMAIN}{obj.token['icon']}"
            }
        return {}

    def get_token(self, obj):
        result = {
            'category': obj.token['category'],
            'symbol': obj.token['symbol'],
            'decimals': obj.token['decimals'] 
        }

        if obj.token['is_nft']:
            result['nfts'] = {
                'description': obj.token['nft_description'],
                'fields': {},
                'parse': {
                    'bytecode': '',
                    'types': obj.token['nft_types']
                }
            }
        
        return result


class BcmrRegistrySerializer(serializers.ModelSerializer):
    version = serializers.SerializerMethodField()
    latestRevision = serializers.SerializerMethodField()
    registryIdentity = serializers.SerializerMethodField()
    identities = serializers.SerializerMethodField()

    class Meta:
        model = Registry
        fields = (
            'version',
            'latestRevision',
            'registryIdentity',
            'identities',
        )

    def get_version(self, obj):
        return {
            'major': obj.major,
            'minor': obj.minor,
            'patch': obj.patch
        }

    def get_latestRevision(self, obj):
        return obj.latest_revision

    def get_registryIdentity(self, obj):
        return {
            'name': obj.name,
            'description': obj.description
        }

    def get_identities(self, obj):
        tokens = obj.tokens.all()
        identities = {}

        for token in tokens:
            token_histories = TokenHistory.objects.filter(token__category=token.category)

            for token_history in token_histories:
                identity = TokenHistorySerializer(token_history)
                timestamp = token_history.date.isoformat()
                identities[token.category] = {}
                identities[token.category][timestamp] = identity.data

        return identities



from bcmr.models import (
    AuthToken,
    TokenHistory,
    Token,
)


def generate_auth_token():
    a = AuthToken()
    a.save()
    return a


def record_token_history(category):
    token = Token.objects.get(category=category)

    icon = ''
    owner = None
    
    if token.icon:
        icon = token.icon.url
    if token.owner:
        owner = token.owner.id

    token_data = {
        'category': token.category,
        'name': token.name,
        'description': token.description,
        'symbol': token.symbol,
        'decimals': token.decimals,
        'icon': icon,
        'is_nft': token.is_nft,
        'nft_description': token.nft_description,
        'nft_types': token.nft_types,
        'status': token.status,
        'owner': owner
    }
    TokenHistory(token=token_data).save()
